package com.sts.businesspanel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONObject;
import android.os.Bundle;
import android.os.Handler;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.view.Menu;
import android.view.View;
import android.widget.EditText;
import com.sts.businesspanel.arquitectura.rest.*;
import com.sts.businesspanel.entities.*;

public class Login extends Activity {

	public final static String DATOS_USUARIO = "com.sts.businesspanel.DATOS_USUARIO";
	public final static String DATOS_CONFIGURACION = "com.sts.businesspanel.DATOS_CONFIGURACION";
	private final static String SERVICE_URI = "http://217.76.135.234:81/BusinessPanelService";
	
	private DatosUsuario datosUsuario;
	private ArrayList<ConfiguracionWidget> datosConfiguracion;
	
	final Handler mHandler = new Handler();
	Boolean isValid = false;
	
	final Runnable mUpdateResults = new Runnable() {
        public void run() {
            updateResultsInUi();
        }
    };
    
    final Runnable mMostrarPanelResult = new Runnable() {
        public void run() {
        	MostrarPanel();
        }
    };
    
    private void updateResultsInUi() {
    	if(isValid){
    		Thread t = new Thread() {
                public void run() {
                	isValid = GetDatosConfiguracion();
                    mHandler.post(mMostrarPanelResult);
                }
            };
            t.start();
    	}
    	else{
    		AlertDialog ad = new AlertDialog.Builder(this, android.R.style.Theme_DeviceDefault_Light_Dialog).create();
    		ad.setCancelable(false); // para bloquear el boton de back del dispositivo
    		ad.setTitle(getString(R.string.login_warning_title));
    		ad.setMessage(getString(R.string.login_warning_mensaje));
    		ad.setButton(
    				DialogInterface.BUTTON_POSITIVE, getString(R.string.Warning_OK), new DialogInterface.OnClickListener() {
		    		    @Override
		    		    public void onClick(DialogInterface dialog, int which) {
		    		        dialog.dismiss();                    
		    		    }
		    		} 
    		);
    		ad.show();
    	}
    }
    
    private void MostrarPanel(){
    	
    	if(isValid){
    		Intent intent = new Intent(this, Panel.class);
    		intent.putExtra(DATOS_USUARIO, datosUsuario);
    		intent.putExtra(DATOS_CONFIGURACION, datosConfiguracion);
    		startActivity(intent);
    	}
    	else{
    		AlertDialog ad = new AlertDialog.Builder(this, android.R.style.Theme_DeviceDefault_Light_Dialog).create();
    		ad.setCancelable(false); // para bloquear el boton de back del dispositivo
    		ad.setTitle(getString(R.string.login_warning_title));
    		ad.setMessage(getString(R.string.login_warning_mensaje_configuracion));
    		ad.setButton(
    				DialogInterface.BUTTON_POSITIVE, getString(R.string.Warning_OK), new DialogInterface.OnClickListener() {
		    		    @Override
		    		    public void onClick(DialogInterface dialog, int which) {
		    		        dialog.dismiss();                    
		    		    }
		    		} 
    		);
    		ad.show();
    	}
    }
    
    protected void ValidarUsuario() {
        Thread t = new Thread() {
            public void run() {
            	isValid = ObtenerDatosUsuario();
                mHandler.post(mUpdateResults);
            }
        };
        t.start();
    }
    
    private Boolean ObtenerDatosUsuario(){
    	return GetServicio();
    }
    
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.activity_login, menu);
        return true;
    }

    public void loginOnClick(View v) {
    	ValidarUsuario();
	}
    
    private Boolean GetServicio() {
    	
    	Boolean datosCorrectos = false;
    	try {
            RESTClient rClient = new RESTClient(SERVICE_URI);
            Map<String,String> params = new HashMap<String, String>();
            EditText usuario = (EditText)findViewById(R.id.etUsuario);
        	EditText password = (EditText)findViewById(R.id.etPassword);
        	
            params.put("u",usuario.getText().toString());
            params.put("p",password.getText().toString());
            
            String resultado = rClient.webGet("ObtenerDatosUsuario", params);
            
            if(!resultado.isEmpty()){
            	datosCorrectos = true;
            	datosUsuario = new DatosUsuario();
            	JSONObject jobj = new JSONObject(resultado);
            	datosUsuario.setId(jobj.getInt("Id"));
            	datosUsuario.setNombre(jobj.getString("Nombre"));
            	datosUsuario.setApellido1(jobj.getString("Apellido1"));
            	datosUsuario.setApellido2(jobj.getString("Apellido2"));
            	datosUsuario.setEmpresa(jobj.getString("Empresa"));
            	datosUsuario.setUsuario(usuario.getText().toString());
            	datosUsuario.setPassword(password.getText().toString());
            }
            
        } catch(Exception e){
            e.toString();
        }
    	
    	return datosCorrectos;
	}
    
    private Boolean GetDatosConfiguracion(){
    	Boolean datosCorrectos = false;
    	try {
            RESTClient rClient = new RESTClient(SERVICE_URI);
            Map<String,String> params = new HashMap<String, String>();
            EditText usuario = (EditText)findViewById(R.id.etUsuario);
        	EditText password = (EditText)findViewById(R.id.etPassword);
        	
            params.put("u",usuario.getText().toString());
            params.put("p",password.getText().toString());
            
            String resultado = rClient.webGet("ObtenerConfiguracionWidgets", params);
            
            if(!resultado.isEmpty()){
            	datosCorrectos = true;
            	JSONArray jobj = new JSONArray(resultado);
            	datosConfiguracion = new ArrayList<ConfiguracionWidget>();
            	
            	for (int i = 0; i < jobj.length(); ++i) {
            		ConfiguracionWidget cw = new ConfiguracionWidget();
            		
            		cw.setId(jobj.getJSONObject(i).getInt("Id"));
            		cw.setNombreControl(jobj.getJSONObject(i).getString("NombreControl"));
            		cw.setTitulo(jobj.getJSONObject(i).getString("Titulo"));
            		cw.setOrden(jobj.getJSONObject(i).getInt("Orden"));
            		
            		datosConfiguracion.add(cw);
            	}
            }
            
        } catch(Exception e){
            e.toString();
        }
    	return datosCorrectos;
    }
}
