package com.sts.businesspanel.customcontrols;

import com.sts.businesspanel.R;
import com.sts.businesspanel.entities.BuscadorEntity;
import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Typeface;
import android.graphics.Paint.Align;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;

public class Buscador extends View {

	//Valores por defecto del control
	private final static int DEFAULT_WIDTH = 400;
	private final static int DEFAULT_HEIGHT = 250;
	private int Id;
	
	private String title;
	private String Valor1;
	private String Valor2;
	private String Valor3;
	private String Valor4;
	private Paint rectanglePaint;
	private Paint textTitlePaint;
    private Paint textSubtitlePaint;
    private Paint textValorPaint;
    private Paint imagePaint;
    private Bitmap imageBuscar;
	
    private OnTouchBuscadorListener listener;
    private OnDragBuscadorListener dragListener;
	
	public interface OnTouchBuscadorListener {
        void onTouchBuscadorOccurred(View v);
	}
	
	public interface OnDragBuscadorListener{
		void OnDragBuscadorOccurred(View v, int id);
	}
	
	public void setOnTouchBuscadorListener(OnTouchBuscadorListener l) {
        listener = l;
	}
	
	public void setOnDragBuscadorListener(OnDragBuscadorListener l) {
		dragListener = l;
	}
	
	@Override
	public int getId() {
		// TODO Auto-generated method stub
		return Id;
	}

	@Override
	public void setId(int id) {
		// TODO Auto-generated method stub
		Id = id;
	}
    
	public void SetValores(String valor1, String valor2, String valor3, String valor4){
		Valor1 = valor1;
		Valor2 = valor2;
		Valor3 = valor3;
		Valor4 = valor4;
		invalidate();
    	requestLayout();
	}
	
	public Buscador(Context ctx, BuscadorEntity conf) {
		super(ctx);
		
		title = conf.getBuscadorTitulo();		
		Valor1 = "";
		Valor2 = "";
		Valor3 = "";
		Valor4 = "";
		Init();
	}
	
	public Buscador(Context ctx, AttributeSet attrs) {
        super(ctx, attrs);
        
    	TypedArray a = ctx.getTheme().obtainStyledAttributes(attrs, R.styleable.Buscador, 0, 0);

    	try {
    		//TODO: Aqui setear las propiedades por ejemplo:
    		//mShowText = a.getBoolean(R.styleable.PieChart_showText, false);
    	} finally {
    		a.recycle();
    	}
    	
    	Init();
    }
	
	@Override
	public boolean onTouchEvent(MotionEvent event) {
		// Obtener la posicion relativa
		float x = event.getRawX() - this.getLeft();
		float y = event.getRawY() - this.getTop();
		
		if(y > 100 && y <= 250 && x <= 400){
			listener.onTouchBuscadorOccurred(this);
		}
		else if(y >= 0 && y <=99 && x <= 400){
			dragListener.OnDragBuscadorOccurred(this, Id);
		}
		
		return super.onTouchEvent(event);
	}
	
	private void Init(){
		setMeasuredDimension(DEFAULT_WIDTH, DEFAULT_HEIGHT);
		
		rectanglePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		rectanglePaint.setStyle(Paint.Style.FILL);
		rectanglePaint.setColor(Color.WHITE);
		
		Typeface tf = Typeface.create(Typeface.SERIF,Typeface.BOLD);
		textTitlePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		textTitlePaint.setStyle(Paint.Style.FILL);
		textTitlePaint.setColor(Color.GRAY);
		textTitlePaint.setTextSize(22);
		textTitlePaint.setTypeface(tf);
		textTitlePaint.setTextAlign(Align.CENTER);
		
		textSubtitlePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		textSubtitlePaint.setStyle(Paint.Style.FILL);
		textSubtitlePaint.setColor(Color.GRAY);
		textSubtitlePaint.setTextSize(18);
		textSubtitlePaint.setTypeface(tf);
		textSubtitlePaint.setTextAlign(Align.LEFT);
		
		Typeface tfv = Typeface.create(Typeface.SERIF,Typeface.NORMAL);
		textValorPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		textValorPaint.setStyle(Paint.Style.FILL);
		textValorPaint.setColor(Color.GRAY);
		textValorPaint.setTextSize(16);
		textSubtitlePaint.setTypeface(tfv);
		textSubtitlePaint.setTextAlign(Align.LEFT);
		
		imagePaint = new Paint();
		imageBuscar = BitmapFactory.decodeResource(getResources(), R.drawable.search);
	}
	
	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		
		canvas.drawRect(0, 0, 400, 250, rectanglePaint);
		canvas.drawText(title, 200, 30, textTitlePaint);
		
		canvas.drawText("Nombre: ", 20, 90, textSubtitlePaint);
		canvas.drawText("Tienda: ", 20, 130, textSubtitlePaint);
		canvas.drawText("Mail: ", 20, 170, textSubtitlePaint);
		canvas.drawText("Telefono: ", 20, 210, textSubtitlePaint);
		
		canvas.drawText(Valor1, 120, 90, textValorPaint);
		canvas.drawText(Valor2, 120, 130, textValorPaint);
		canvas.drawText(Valor3, 120, 170, textValorPaint);
		canvas.drawText(Valor4, 120, 210, textValorPaint);
		
        canvas.drawBitmap(imageBuscar, 330, 0, imagePaint);
	}	
}
