package com.sts.businesspanel.customcontrols;

import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import org.json.JSONArray;
import org.json.JSONException;
import com.sts.businesspanel.R;
import com.sts.businesspanel.arquitectura.rest.RESTClient;
import com.sts.businesspanel.entities.DatosGrafico;
import com.sts.businesspanel.entities.GraficoEntity;
import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Typeface;
import android.graphics.Paint.Align;
import android.os.Handler;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;

public class Grafico extends View{

	//Valores por defecto del control
	private final static int DEFAULT_WIDTH = 400;
	private final static int DEFAULT_HEIGHT = 250;
	private final static int EJEX_WIDTH = 310;
	private final static int EJEY_HEIGHT = 170;
	private final static int PORCIONES_Y = 4;
	private final static int POSICION_INICIAL_Y = 220;
	private final static int POSICION_INICIAL_X = 50;
	private int Id;
	
	private int escala;
	private DatosGrafico[] datos;
	private String title;
	private float yMaxValue;
	private float yParticiones;
	private float yEscala;
	private float xParticiones;
	private Boolean autoRefresh = true;
	
	//Datos servicio
	private String usuario;
	private String password;
	private String urlServicio;
	private String metodo;
	private long MiliseconsUpdateInterval;
	private long MiliseconsStartInterval;
	private Timer t;
	private Handler mHandler;
	
	//Paints
	private Paint ejesPaint;
    private Paint lineaPaint;
    private Paint textTitlePaint;
    private Paint textEjeXPaint;
    private Paint textEjeYPaint;
    private Paint rectanglePaint;
    
    private OnTouchGraficoListener listener;
    private OnDragGraficoListener dragListener;
	
	public interface OnTouchGraficoListener {
        void onTouchGraficoOccurred(View v);
	}
	
	public interface OnDragGraficoListener{
		void OnDragGraficoOccurred(View v, int id);
	}
	
	public void setOnTouchGraficoListener(OnTouchGraficoListener l) {
        listener = l;
	}
	
	public void setOnDragGraficoListener(OnDragGraficoListener l) {
		dragListener = l;
	}
	
	@Override
	public int getId() {
		// TODO Auto-generated method stub
		return Id;
	}

	@Override
	public void setId(int id) {
		// TODO Auto-generated method stub
		Id = id;
	}

	public Grafico(Context ctx, GraficoEntity conf, DatosGrafico[] d) {
		super(ctx);
		
		title = conf.getTitle();
		escala = conf.getEscala();
		datos = d;
		usuario = conf.getUsuario();
		password = conf.getPassword();
		urlServicio = conf.getUrlServicio();
		metodo = conf.getMetodo();
		MiliseconsUpdateInterval = conf.getMiliseconsUpdateInterval();
		MiliseconsStartInterval = conf.getMiliseconsStartInterval();
		autoRefresh = conf.getAutoRefresh();
		
		Init();
	}
	public Grafico(Context ctx, GraficoEntity conf) {
		super(ctx);
		
		title = conf.getTitle();
		escala = conf.getEscala();
		
		usuario = conf.getUsuario();
		password = conf.getPassword();
		urlServicio = conf.getUrlServicio();
		metodo = conf.getMetodo();
		MiliseconsUpdateInterval = conf.getMiliseconsUpdateInterval();
		MiliseconsStartInterval = conf.getMiliseconsStartInterval();
		mHandler = new Handler();
		t = new Timer();
		autoRefresh = conf.getAutoRefresh();
		
		Init();
	}
	
	public Grafico(Context ctx, AttributeSet attrs) {
        super(ctx, attrs);
        
    	TypedArray a = ctx.getTheme().obtainStyledAttributes(attrs, R.styleable.Contador, 0, 0);

    	try {
    		//TODO: Aqui setear las propiedades por ejemplo:
    		//mShowText = a.getBoolean(R.styleable.PieChart_showText, false);
    	} finally {
    		a.recycle();
    	}
    	
    	Init();
    }
	
	private void Init(){
		setMeasuredDimension(DEFAULT_WIDTH * escala, DEFAULT_HEIGHT * escala);
		
		rectanglePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		rectanglePaint.setStyle(Paint.Style.FILL);
		rectanglePaint.setColor(Color.WHITE);
		
		Typeface tf = Typeface.create(Typeface.SERIF,Typeface.BOLD);
		textTitlePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		textTitlePaint.setStyle(Paint.Style.FILL);
		textTitlePaint.setColor(Color.GRAY);
		textTitlePaint.setTextSize(22);
		textTitlePaint.setTypeface(tf);
		textTitlePaint.setTextAlign(Align.CENTER);
		
		textEjeXPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		textEjeXPaint.setStyle(Paint.Style.FILL);
		textEjeXPaint.setColor(Color.GRAY);
		textEjeXPaint.setTextSize(12);
		textEjeXPaint.setTypeface(tf);
		
		textEjeYPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		textEjeYPaint.setStyle(Paint.Style.FILL);
		textEjeYPaint.setColor(Color.GRAY);
		textEjeYPaint.setTextSize(12);
		textEjeYPaint.setTypeface(tf);
		
		ejesPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		ejesPaint.setStyle(Paint.Style.FILL);
		ejesPaint.setColor(Color.GRAY);
		
		lineaPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		lineaPaint.setStyle(Paint.Style.FILL);
		lineaPaint.setColor(Color.RED);
		lineaPaint.setStrokeWidth(3);
		
		if(autoRefresh){
			t.scheduleAtFixedRate(new UpdateTimeTask(), MiliseconsStartInterval, MiliseconsUpdateInterval);	
		}
	}
	
	final Runnable Refresh = new Runnable() {
        public void run() {
        	invalidate();
        	requestLayout();
        }
    };
	
	class UpdateTimeTask extends TimerTask {
	   public void run() {
		   
		   RESTClient rClient = new RESTClient(urlServicio);
		   Map<String,String> params = new HashMap<String, String>();
		   params.put("u",usuario);
           params.put("p",password);
           
           String resultado = rClient.webGet(metodo, params);
           
           if(!resultado.isEmpty()){
        	   try {
        		   JSONArray jobj = new JSONArray(resultado);
        		   datos = new DatosGrafico[jobj.length()];
	               	
	               for (int i = 0; i < jobj.length(); ++i) {	               		
		               DatosGrafico dg = new DatosGrafico();
		               dg.setEjeX(jobj.getJSONObject(i).getString("Mes"));
		               dg.setEjeY(jobj.getJSONObject(i).getInt("Valor"));
	                   datos[i] = dg;    
	               }
	               
	               
	               
        	   } catch (JSONException e) {
					e.toString();
        	   }
           }
           
           mHandler.post(Refresh);
	   }
	}
	
	@Override
	public boolean onTouchEvent(MotionEvent event) {
		if(escala == 1){
			// Obtener la posicion relativa
			float x = event.getRawX() - this.getLeft();
			float y = event.getRawY() - this.getTop();
			
			if(y > 100 && y <= 250 && x <= 400){
				listener.onTouchGraficoOccurred(this);
			}
			else if(y >= 0 && y <=99 && x <= 400){
				dragListener.OnDragGraficoOccurred(this, Id);
			}
		}
		return super.onTouchEvent(event);
	}
	
	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		
		canvas.drawRect(0, 0, DEFAULT_WIDTH * escala, DEFAULT_HEIGHT * escala, rectanglePaint);
		
		canvas.drawText(title, 200 * escala, 30 * escala, textTitlePaint);
		
		//eje horizontal
		canvas.drawLine(50 * escala, POSICION_INICIAL_Y * escala, 360 * escala, POSICION_INICIAL_Y * escala, ejesPaint);

		//eje vertical
		canvas.drawLine(50 * escala, 50 * escala, 50 * escala, 220 * escala, ejesPaint);
		
		if(datos != null && datos.length > 0){
			
			// Obtenemos el valor maximo del eje y
       		for (int i = 0; i < datos.length; i++) {
       			if(datos[i].getEjeY() > yMaxValue){
       				yMaxValue = datos[i].getEjeY();
       			}
       		}
       		
       		//Obtenemos el numero de particiones del ejex
       		xParticiones = (float)(EJEX_WIDTH * escala) / (float)datos.length;
       		
       		//Obtenemos el numero de particiones del ejey
       		yParticiones = (float) (EJEY_HEIGHT * escala) / (float) yMaxValue;
       		
       		//Para mostrar el eje y como pueden ser valores muy grandes en modo widget lo divido el 4 valores
       		yEscala = (float) yMaxValue / (float) (PORCIONES_Y * escala);
       		
			//Pinto los valores del ejey
			float yPosicionInicial = (float)(POSICION_INICIAL_Y * escala);
			float indicadorY = 0;
			for (int i = 0; i < (PORCIONES_Y * escala); i++) {
				indicadorY += yEscala;
				yPosicionInicial -= ((float)(EJEY_HEIGHT * escala) / (float)(PORCIONES_Y * escala));
				canvas.drawText(String.valueOf((int)indicadorY), POSICION_INICIAL_X * escala - 30, yPosicionInicial, textEjeYPaint);
				
			}
			
			//Pinto los valores del ejex
			float xPosicionInicial = (float)POSICION_INICIAL_X * escala;
			for (int i = 0; i < datos.length; i++) {
				xPosicionInicial += xParticiones;
				canvas.drawText(datos[i].getEjeX(), xPosicionInicial, (float)(POSICION_INICIAL_Y * escala) + (float)20, textEjeXPaint);
			}
			
			
			//Pinto los valores del grafico se asume que los valores estn ordenados de mayor a menor
			xPosicionInicial = (float)POSICION_INICIAL_X * escala;
			yPosicionInicial = (float)POSICION_INICIAL_Y * escala;
			float xPosicionFinal = (float)POSICION_INICIAL_X * escala;
			float yPosicionFinal = (float)POSICION_INICIAL_Y * escala;
			
			for (int i = 0; i < datos.length; i++) {
				if(i > 0){
					xPosicionInicial = xPosicionFinal;
					yPosicionInicial = yPosicionFinal;
				}
				
				xPosicionFinal += xParticiones;
				yPosicionFinal = ((float)POSICION_INICIAL_Y * escala - (float)datos[i].getEjeY() * yParticiones);
				
				canvas.drawLine(xPosicionInicial, yPosicionInicial, xPosicionFinal, yPosicionFinal, lineaPaint);
			}
		}
	}

	public int getEscala() {
		return escala;
	}

	public void setEscala(int escala) {
		this.escala = escala;
	}

	public DatosGrafico[] getDatos() {
		return datos;
	}

	public void setDatos(DatosGrafico[] datos) {
		this.datos = datos;
	}	
	
}
