using System;
using System.Collections.Generic;
using System.Data.Linq;
using System.Linq;
using BusinessPanelDatos.Entities.Enums;

namespace BusinessPanelDatos.Repository.BaseEntities
{
    public class BaseRepository<TContext> where TContext : DataContext, new()
    {
        public TContext context;

        public BaseRepository()
        {
            context = new TContext();
        }

        public virtual void UpdateChanges<T, G>(T updateList, Type type)
            where T : List<G>
            where G : BaseEntidad
        {
            using (TContext contextToUpdate = new TContext())
            {
                ITable table = contextToUpdate.GetTable(type);

                if (updateList.Select(x => x.RowState == RowState.Modified).Count() > 0)
                {
                    IEnumerable<G> modified = updateList.Where<G>(entidad => entidad.RowState == RowState.Modified);
                    table.AttachAll(modified, true);
                }

                if (updateList.Select(x => x.RowState == RowState.Deleted).Count() > 0)
                {
                    table.AttachAll(updateList.Where<G>(entidad => entidad.RowState == RowState.Deleted), false);
                    table.DeleteAllOnSubmit(updateList.Where<G>(entidad => entidad.RowState == RowState.Deleted));
                }

                if (updateList.Select(x => x.RowState == RowState.Added).Count() > 0)
                {
                    table.InsertAllOnSubmit(updateList.Where<G>(entidad => entidad.RowState == RowState.Added));
                }

                contextToUpdate.SubmitChanges();
            }
        }

        public virtual void UpdateChanges<T, G>(T updateList)
            where T : List<G>
            where G : BaseEntidad
        {
            UpdateChanges<T, G>(updateList, updateList[0].GetType());
        }
    }
}