﻿using System;
using System.Collections.Generic;
using System.Data.Linq;
using System.Globalization;
using BusinessPanelDatos.Entities;
using BusinessPanelDatos.Entities.Enums;
using BusinessPanelDatos.Linq;
using BusinessPanelDatos.Repository.BaseEntities;
using System.Linq;

namespace BusinessPanelDatos.Repository
{
    public class BusinessPanelRepostory : BaseRepository<BusinessPanelDataContext>
    {
        public bool ValidarUsuario(string u, string p)
        {
            var query = (from usuario in context.D_Usuarios
                            join pu in context.D_UsuariosPermisos on usuario.Id equals pu.IdUsuario 
                            join permiso in context.C_Permisos on pu.IdPermiso equals permiso.Id
                         where usuario.Login.ToLower() == u.ToLower() &&
                               usuario.Password.ToLower() == p.ToLower() &&
                               pu.IdPermiso == (int)Permisos.AccesoPFC &&
                               permiso.IdAplicacion == (int)Aplicaciones.PFC
                         select usuario);

            return query.Any();
        }

        public DatosUsuario ObtenerDatosUsuario(string u, string p)
        {
            DatosUsuario datos = null;
            var resultado = context.D_Usuarios.SingleOrDefault(x => x.Login.ToLower() == u.ToLower() && 
                                                                    x.Password.ToLower() == p.ToLower());
            
            if (resultado != null)
            {
                datos = new DatosUsuario
                {
                    Apellido1 = resultado.Apellido1,
                    Apellido2 = resultado.Apellido2,
                    Empresa = resultado.Empresa,
                    Id = resultado.Id,
                    Nombre = resultado.Nombre
                };
            }
            
            return datos; 
        }

        public ContadorSimpleEntity ObtenerDisponibilidadComerciales(DateTime fecha)
        {
            var totalEmpleados = context.D_Empleados.Count();
            var totalRegistrados = context.D_RegistroEmpleados.Count(x => x.FechaRegistro.Year == fecha.Year && 
                                                                          x.FechaRegistro.Month == fecha.Month && 
                                                                          x.FechaRegistro.Day == fecha.Day);

            return new ContadorSimpleEntity
            {
                Valor1 = totalRegistrados,
                Valor2 = (totalEmpleados - totalRegistrados)
            };
        }

        public DatosComercial[] ObtenerComercialesConectados()
        {
            return context.D_Obtener_Comerciales_Conectados(DateTime.Now)
                            .Select(x => new DatosComercial
                            {
                                Mail = x.Email,
                                Nombre = String.Format("{0} {1} {2}", x.Nombre, x.Apellido1, x.Apellido2),
                                Telefono = x.Movil.HasValue ? x.Movil.Value.ToString(CultureInfo.InvariantCulture) : String.Empty,
                                Tienda = x.Descripcion
                            }).ToArray();
        }

        public DatosComercial[] ObtenerComercialesBaja()
        {
            return context.D_Obtener_Comerciales_Baja(DateTime.Now)
                            .Select(x => new DatosComercial
                            {
                                Mail = x.Email,
                                Nombre = String.Format("{0} {1} {2}", x.Nombre, x.Apellido1, x.Apellido2),
                                Telefono = x.Movil.HasValue ? x.Movil.Value.ToString(CultureInfo.InvariantCulture) : String.Empty,
                                Tienda = x.Descripcion
                            }).ToArray();
        }

        public ContadorSimpleEntity ObtenerVentasTotalesTiendas(DateTime fecha)
        {
            var totalTiendasConVentas = context.D_Obtener_Tiendas_Con_Ventas(fecha).Count();
            var totalTiendasSinVentas = context.D_Obtener_Tiendas_Sin_Ventas(fecha).Count();

            return new ContadorSimpleEntity
            {
                Valor1 = totalTiendasConVentas,
                Valor2 = totalTiendasSinVentas
            };
        }

        public DatosTienda[] ObtenerTiendasConVentas()
        {
            return context.D_Obtener_Tiendas_Con_Ventas(DateTime.Now)
                            .Select(x => new DatosTienda
                                {
                                    Mail = x.Email,
                                    Nombre = x.Descripcion,
                                    Telefono = x.Movil.HasValue ? x.Movil.Value.ToString(CultureInfo.InvariantCulture) : String.Empty,
                                    Responsable = String.Format("{0} {1} {2}", x.Nombre, x.Apellido1, x.Apellido2) 
                                }).ToArray();
        }

        public DatosTienda[] ObtenerTiendasSinVentas()
        {
            return context.D_Obtener_Tiendas_Sin_Ventas(DateTime.Now)
                            .Select(x => new DatosTienda
                            {
                                Mail = x.Email,
                                Nombre = x.Descripcion,
                                Telefono = x.Movil.HasValue ? x.Movil.Value.ToString(CultureInfo.InvariantCulture) : String.Empty,
                                Responsable = String.Format("{0} {1} {2}", x.Nombre, x.Apellido1, x.Apellido2)
                            }).ToArray();
        }

        public DatosComercial[] BuscarComercial(string pattern)
        {
            return context.D_Buscar_Comercial(pattern).Select(x => new DatosComercial
            {
                Mail = x.Email,
                Nombre = String.Format("{0} {1} {2}", x.Nombre, x.Apellido1, x.Apellido2),
                Telefono = x.Movil.HasValue ? x.Movil.Value.ToString(CultureInfo.InvariantCulture) : String.Empty,
                Tienda = x.TiendaDescripcion
            }).ToArray();
        }

        public Ventas[] ObtenerVentasPorMes(DateTime fecha)
        {
            return context.D_Obtener_Ventas_Mes_Año(fecha)
                            .Select(x => x.TotalVentas != null ? new Ventas
                            {
                                Mes = x.Mes.HasValue ? x.Mes.Value.ToString(CultureInfo.InvariantCulture) : string.Empty,
                                Valor = (int) x.TotalVentas
                            } : null).ToArray();
        }

        public Widget[] ObtenerConfiguracionWidgets(string u, string p)
        {
            var resultado = from usuario in context.D_Usuarios
                            join conf in context.D_UsuarioConfiguracion on usuario.Id equals conf.IdUsuario
                            join widget in context.D_Widget on conf.IdWidget equals widget.Id
                            join wdesc in context.C_TipoWidget on widget.Tipo equals wdesc.Id
                            where usuario.Login.ToLower().Equals(u.ToLower()) &&
                                  usuario.Password.ToLower().Equals(p.ToLower()) &&
                                  conf.IdAplicacion == (int) Aplicaciones.PFC &&
                                  widget.IdAplicacion == (int) Aplicaciones.PFC
                            select new Widget
                            {
                                Id = widget.Id,
                                NombreControl = widget.NombreControl,
                                Titulo = widget.Titulo,
                                Orden = conf.Orden
                            };

            return resultado.ToArray();
        }

        public void GuardarConfiguracion(string u, string p, Dictionary<int, int> orden)
        {
            var usuario = ObtenerDatosUsuario(u, p);

            var listadoActualizar = (from x in context.D_UsuarioConfiguracion where x.IdUsuario == usuario.Id select x).ToList();

            foreach (var item in listadoActualizar.Where(item => orden.ContainsKey(item.IdWidget)))
            {
                item.Orden = orden[item.IdWidget];
            }

            context.SubmitChanges(ConflictMode.FailOnFirstConflict);
        }

        #region Carga de datos para la demo

        public void CargarDatosDemo()
        {
            var re = new List<D_RegistroEmpleados>
            {
                new D_RegistroEmpleados
                {
                    IdEmpleado = 7,
                    IdTienda = 1,
                    FechaRegistro = DateTime.Now
                },
                new D_RegistroEmpleados
                {
                    IdEmpleado = 9,
                    IdTienda = 1,
                    FechaRegistro = DateTime.Now
                }
            };

            context.D_RegistroEmpleados.InsertAllOnSubmit(re);

            var tv = new List<D_Tienda_Ventas>
            {
                new D_Tienda_Ventas
                {
                    IdTienda = 1,
                    Fecha = DateTime.Now,
                    TotalVentas = 10
                },
                new D_Tienda_Ventas
                {
                    IdTienda = 2,
                    Fecha = DateTime.Now,
                    TotalVentas = 10
                }
            };

            context.D_Tienda_Ventas.InsertAllOnSubmit(tv);

            context.SubmitChanges(ConflictMode.FailOnFirstConflict);
        }

        #endregion
    }
}